function [xyz,f,freq,flag] = Newton_trust_region(xyz,tol,iter_max)

Na = size(xyz,2);
xyz1 = xyz;
x = remove_rotations_translations(xyz);
N = length(x);

Rmax = 0.1;
Rmin = 1e-14;
rho_good = 0.75;
rho_bad = 0.25;
eta = 0.01;
iiimax = 1000;

[f, g] = LJ(x);
nor = norm(g);
R = Rmax/5; % initial trust region radius

% fprintf('Initially, V = %d\t, |grad V| = %d\n',f,nor); 
% fig = figure;
% drawLJconf(xyz,fig);
%% The trust region BFGS method
tic

iter = 0;
flag = 1;
I = eye(length(x));
while nor > tol & iter < iter_max

    % solve the constrained minimization problem using dogleg strategy
    B = LJhess(x);
    if cond(B) > 1e14
        p = cauchy_point(B,g,R);
    else
        [evec,ev] = eig(B);
        [eval,isort] = sort(diag(ev),'ascend');
        evec = evec(:,isort);
        pstar = -B\g;
        if norm(pstar) <= R & eval(1) > 0
            p = pstar;
        else % solve constrained minimization problem
            if abs(g'*evec(:,1)) > 1e-12
                lam = max(0,-eval(1)) + 1;
                isub = 0;
                iii = 0;
                while iii < iiimax
                    B1 = B + lam*I;
                    C = chol(B1);
                    p = -C\(C'\g);
                    np = norm(p);
                    dd = abs(np - R);
                    if dd < 1e-6
                        break
                    end
                    q = C'\p;
                    nq = norm(q);
                    lamnew = lam +(np/nq)^2*(np - R)/R;
                    if lamnew < max(0,-eval(1))
                        lam = 0.5*(lam + max(0,-eval(1)));
                    else
                        lam = lamnew;
                    end
                    isub = isub + 1;
%                     fprintf('isub = %d\n',isub);
                    iii = iii + 1;
                end
            else
                p = cauchy_point(B,g,R);
            end
            if iii == iiimax
                p = cauchy_point(B,g,R);
            end
        end
    end
    iter = iter + 1;  
    if flag == 0
        break;
    end

    % assess the progress
    xnew = x + p;
    [fnew, gnew] = LJ(xnew);
    mnew = f + g'*p + 0.5*p'*B*p;
    rho = (f - fnew+1e-14)/(f - mnew+1e-14);


    % adjust the trust region
    if rho < rho_bad
        R = max([0.25*R,Rmin]);
    else
        if rho > rho_good & abs(norm(p) - R) < tol
            R = min([Rmax,2*R]);
        end
    end
    % accept or reject step
    if rho > eta            
        x = xnew;
        f = fnew;
        g = gnew;
        nor = norm(g);

%         fprintf('iter # %d: f = %.14f, |df| = %.4e, rho = %.4e, R = %.4e\n',iter,f,nor,rho,R);

%         x_aux = [0;0;0;x(1);0;0;x(2:3);0;x(4:N)];
%         xyz = reshape(x_aux,3,Na);        
%         drawLJconf(xyz,fig); drawnow;

    end
    if cond(B) > 1e12
        B = I;
    end
end
if iter == iter_max
    flag = 0;
end
cputime = toc;
%    fprintf('CPUtime = %d, iter = %d\n',cputime,iter);
%% check what is found
% x_aux = [0;0;0;x(1);0;0;x(2:3);0;x(4:N)];
% xyz = reshape(x_aux,3,Na); 
% drawLJconf(xyz,fig); drawnow;

if flag == 1
    H = LJhess(x);
    ihess = sort(eig(H),'ascend');
    freq = sum(log(abs(ihess)));
    if ihess(1) > 0 % if a nondegenerate minimum is found
        flag = 1;
        MorseIndex = 0;
    else
        flag = -1;
    end
    x_aux = [0;0;0;x(1);0;0;x(2:3);0;x(4:N)];
    xyz = reshape(x_aux,3,Na);
else
    fprintf('Newton trust region method failed to find a minimum\n');
    xyz = xyz1;
    freq = 0;
    flag = 0;
end

end
  %% Lennard-Jones potential        
function [v dv] = LJ(xyz) % 
global feval
feval = feval + 1;
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates 
x = reshape(x_aux,3,Na);
r = zeros(Na);
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1));
end
r = r + diag(ones(Na,1));
aux = 1./r.^6;
L = (aux - 1).*aux;
L = L - diag(diag(L));
v = 2*sum(sum(L));

g = zeros(size(x_aux));
L = -6*(2./r.^6 - 1)./r.^8;
for k = 1 : Na
    Lk = L(:,k);
    g(1 + (k-1)*3 : k*3) = (x(:,k)*ones(1,Na) - x)*Lk;
end

g = 4*g;
dv = g([4,7,8,10 : Na*3]);

end

%%
function v = LJpot(xyz) % 
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates 
x = reshape(x_aux,3,Na);
r = zeros(Na);
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1));
end
r = r + diag(ones(Na,1));
aux = 1./r.^6;
L = (aux - 1).*aux;
L = L - diag(diag(L));
v = 2*sum(sum(L));
end
 
 %%
function dv = LJgrad(xyz) % 
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates
g = zeros(size(x_aux));
x = reshape(x_aux,3,Na);
r = zeros(Na);
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1));
end
r = r + diag(ones(Na,1));
L = -6*(2./r.^6 - 1)./r.^8;
for k = 1 : Na
    Lk = L(:,k);
    g(1 + (k-1)*3 : k*3) = (x(:,k)*ones(1,Na) - x)*Lk;
end

g = 4*g;
dv = g([4,7,8,10 : Na*3]);
end

%%
 function dv = LJder(r)
 dv = -12*r.^(-13) + 6*r.^(-7);
 end
%%
%%
function H = LJhess(x) % find the Hessian using finite differences
h = 1e-6;
n = length(x);
H = zeros(n);
e = eye(n);
for i = 1 : n
    di = e(:,i)*h;
    Hei = 0.5*(LJgrad(x + di) - LJgrad(x - di))/h;
    for j = 1 : i
        H(j,i) = e(j,:)*Hei;
        H(i,j) = H(j,i);
    end
end
end

%%
    function p = cauchy_point(B,g,R)
        ng = norm(g);
        ps = -g*R/ng;
        aux = g'*B*g;
        if aux <= 0
            p = ps;
        else
            a = min(ng^3/(R*aux),1);
            p = ps*a;
        end
    end
