function glue_networks()
global fname1 fname1xyz
%This program computes the edges in networks of clusters between size low
%and size high. Each edge corresponds to an attachment probability.

% Developed by Yakir Forman, July 2016


%Name of each output file: filename_x_x+1.txt
%Format of file:
%Index of minimum in x      Index of minimum in x+1     Probability
%Index of minimum in x      Index of minimum in x+1     Probability
%etc.
Na = 11;
Na1 = Na + 1;
dir0 = sprintf('LJ%ddata',Na);
fname0 = strcat(dir0,sprintf('/LJ%dmin_data.txt',Na));
fname0xyz = strcat(dir0,sprintf('/LJ%dmin_xyz.txt',Na));
dir1 = sprintf('LJ%ddata',Na1);
fname1 = strcat(dir1,sprintf('/LJ%dmin_data.txt',Na1));
fname1xyz = strcat(dir1,sprintf('/LJ%dmin_xyz.txt',Na1));

fname = sprintf('Connect%dand%d.txt',Na,Na1);

jstart = 1; % the minimum of LJN that you start with in gluing LJN to LJN+1

%Tolerance to separate between different minima:
min_tol = 1e-6;

    minxyz0 = load(fname0xyz); %Coordinates of minima for size Na
    mindata1 = load(fname1); %Data for minima for size Na + 1
    ff = fopen(fname,'a');
    [Nmin1 mx] = size(mindata1);
    if mx~=3
        fprintf('Warning: %s file does not have 3 columns\n',fname1);
        return;
    end
    minvals1 = mindata1(:,1); %Values of minima for size Na + 1
    min_dif = minvals1(2 : Nmin1) - minvals1(1 : Nmin1 - 1);
%     if min(min_dif) < 2*min_tol
%         fprintf('Warning: tolerance is too high\n');
%         return;
%     end
    [l lx] = size(minxyz0);
    if lx ~= Na
        fprintf('Warning: %s file does not have %d columns\n',fname0xyz,Na);
        return;
    end
    if mod(l,3) ~= 0
        fprintf('Warning: LJ%dmin_xyz.txt file does not have a number of rows divisible by 3\n',x);
        return;
    end
    Nmin0 = l/3;
    for j = jstart : Nmin0
        fprintf('glue minimum %d of LJ%d\n',j,Na);
        xyz = minxyz0(3*(j - 1) + 1 : 3*j,:); %Read configuration for minimum j
        [minima prob minvals1] = add_atom_iso(xyz,minvals1); %Compute attachment probabilities
        for k = 1 : length(minima)
            fprintf(ff,'%d\t %d\t %.4e\n',j,minima(k),prob(k));
            fprintf('%d\t %d\t %.4e\n',j,minima(k),prob(k));
        end
    end
    fclose(ff);
end

%%

function [minima, prob, minvals] = add_atom_iso(xyz,minvals)
global fname1 fname1xyz

%
% Yakir Forman
% July 12, 2016

% minvals1 = LJpot(minima of cluster with Na + 1 atoms)

[m Na] = size(xyz);
%% Parameters:
min_tol = 1e-6; % Tolerance to separate between different minima:
tol = 1e-9;
iter_max = 500;
Rmax = 0.1;

isovalue = -0.1;
rstar = 2^(1/6);

nf_target = 1000; % the desired number of faces


fac = 0.1; % parameter defining the coarseness of meshgrid

min_xyz = min(min(xyz));
max_xyz = max(max(xyz));
step = fac*(max_xyz - min_xyz);
mi = min(xyz,[],2) - 20*step;
ma = max(xyz,[],2) + 20*step;
[X Y Z] = meshgrid(mi(1):step:ma(1),mi(2):step:ma(2),mi(3):step:ma(3));

%% Compute energy 
e = ones(size(X));
d = zeros(size(X));
V = zeros(size(X));
R = rstar*e;
for k = 1 : Na
    dx = X - xyz(1,k)*e;
    dy = Y - xyz(2,k)*e;
    dz = Z - xyz(3,k)*e;
    d = sqrt(dx.^2 + dy.^2 + dz.^2);
    d = max(R,d); %
    d6 = 1./d.^6;
    V = V + (d6 - e).*d6;
end
V = 4*V;



[faces0 verts0] = isosurface(X,Y,Z,V,isovalue);
f0 = size(faces0,1);
if f0 > nf_target
    rfac = nf_target/size(faces0,1);
    [faces,verts] = reducepatch(faces0,verts0,rfac);
else
    faces = faces0;
    verts = verts0;
end

nf = size(faces,1); % Number of faces
fprintf('nfaces = %d\n',nf);

centers = [sum([verts(faces(:,1),1),verts(faces(:,2),1),verts(faces(:,3),1)],2)/3,...
        sum([verts(faces(:,1),2),verts(faces(:,2),2),verts(faces(:,3),2)],2)/3,...
        sum([verts(faces(:,1),3),verts(faces(:,2),3),verts(faces(:,3),3)],2)/3]; % centes of faces
a = verts(faces(:, 2), :) - verts(faces(:, 1), :);
b = verts(faces(:, 3), :) - verts(faces(:, 1), :);
c = cross(a, b, 2);
areas = 0.5*sqrt(sum(c.^2,2)); % areas of faces

nfail = 0;
pr = zeros(size(minvals,1),1);
for i = 1 : nf
    new_atom = centers(i,:)';%+h*ortho(:,i); %New atom is placed in face center
    xyz1 = [xyz new_atom]; %New atom is appended to initial configuration
    [xyz1,f,freq,flag] = BFGS_trust_region(xyz1,tol,iter_max);
    if flag ~= 1 % if BFGS failed, try Newton
        [xyz1,V,freq,flag] = Newton_trust_region([xyz new_atom],tol,iter_max);
        fprintf('Newton: flag = %d\n',flag)
    end
    if flag ~= 1
        nfail = nfail + 1;
        fprintf('Failed to converge to a minimum, nfail = %d\n',nfail);
    else
        [dd, index(i)] = min(abs(minvals - f));
        if dd < min_tol
            pr(index(i)) = pr(index(i)) + areas(i);
        else
            fprintf('Minimum # %d of LJ%d is found: V = %.10f\n',size(minvals,1) + 1, Na + 1,f);
%            drawconf(xyz1);
            minvals = [minvals; f];
            pr = [pr; 0];
            pr(size(pr,1)) = areas(i);
            pgo = point_group_order(xyz1);
            fid = fopen(fname1,'a');
            fprintf(fid,'%.10f\t%.10f\t%d\n',f,freq,pgo);
            fclose(fid);
            fid = fopen(fname1xyz,'a');
            for j = 1 : 3
                fprintf(fid,'%.10f\t',xyz1(j,:)); 
                fprintf(fid,'\n');
            end
            fclose(fid);
        end
    end
end

pr = pr/sum(pr);
minima = find(pr > 0);
prob = pr(minima);

end